// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

library molview_pubchem.module.compounds;

import 'dart:async';
import 'dart:convert';

import 'package:omnibus/omnibus.dart';
import 'package:molview_messages/dataserver.dart';
import 'package:molview_pubchem/molview_pubchem.dart';
import 'package:http/http.dart' as http;
import 'package:molview_messages/standards.dart' as fields;
import 'package:molview_messages/molview_messages_v0.dart' as v0;

part 'information.dart';
part 'retrieve_data.dart';
part 'find_suggestions.dart';

/// HTTP client that is used by this module.
http.BaseClient httpClient;

// All PubChem key properties mapped to their index in the data table.
var keyMap = new Map<String, int>();

// All PubChem value properties mapped to their index in the data table.
var valueMap = new Map<String, int>();

/// Bootstrap PubChem compound table handlers.
void main(Omnibus bus, http.BaseClient _httpClient) {
  httpClient = _httpClient;

  // Generate utility maps.
  for (var i = 0; i < compoundTable.keys.length; i++) {
    keyMap[keyPropDict[compoundTable.keys[i].dataClass]] = i;
  }
  for (var i = 0; i < compoundTable.values.length; i++) {
    valueMap[valuePropDict[compoundTable.values[i].dataClass]] = i;
  }

  // Bind handlers and publish table.
  bus.bind(new v0.RetrieveDataHandler(retrieveData,
      database: 'pubchem', table: 'compounds'));
  bus.bind(new v0.FindSearchSuggestionsHandler(findSuggestions));

  // Activate compounds table in the dataserver.
  bus.event(new v0.ActivateTable('pubchem', 'compounds', compoundTable));
}
